########################################################################
#
# NIC - NMR Integration Calculations 
#
# Author: Robert Boyko
# Place: Department of Biochemistry, University of Alberta
#
########################################################################
 

# define this software
set nicVar(PROG)            	nic
set nicVar(VERSION)         	1.0
set nicVar(VERSION_DATE)    	"Sep 29, 2006"
set nicVar(NAME)            	"$nicVar(PROG)-$nicVar(VERSION)"
set nicVar(WEB_HOME) 			"http://www.bionmr.ualberta.ca/software/nic"

# where to find other files that this software uses
set nicVar(INSTALL_DIR)			"."
set nicVar(HEADER_IMAGE) 		"$nicVar(INSTALL_DIR)/nic.lib/nic.gif"


########################################################################
#
# Variables users may be interested in changing
#

# default input directory
set nicVar(input_dir)			"data"

# default output file (based on input directory choice) 
set nicVar(fileSuffix)			"-nic.out"
set nicVar(output_file)			"[file tail $nicVar(input_dir)]$nicVar(fileSuffix)"

# this is the area we calculate volume
set nicVar(lb)					290.0
set nicVar(ub)					320.0

# set rFlag=1 if you want to see number of points used in the calculations
# in the output
set nicVar(rFlag) 				0	

# main window attributes
set nicVar(main,name)			".$nicVar(PROG)"
set nicVar(main,geometry)		+50+50
set nicVar(main,bg)				"steel blue"
set nicVar(main,fg)				"black"
set nicVar(main,entry,bg)		"beige"
set nicVar(main,listing,bg)		"beige"
set nicVar(main,file_len)		30
set nicVar(main,fixed_font)		"-bitstream-courier-medium-r-normal--0-0-0-0-m-0-iso8859-1"

# Plot attributes
set nicVar(graph,geometry)		600x400
set nicVar(graph,bg)			white
set nicVar(graph,line1_color)	blue
set nicVar(graph,line1_shape)	oval
set nicVar(graph,line1_size)	3
set nicVar(graph,axis_color)	black
set nicVar(graph,bound_color)	red
set nicVar(graph,title_color)	black
set nicVar(graph,axisLimits)	0	
set nicVar(graph,xStart)		90	
set nicVar(graph,yStart)		60	
set nicVar(graph,yLabOffset)	20	
set nicVar(graph,xScaleOffset)	20

# If you want to manually set the x and y axis, these lines can be  
# un-commented and set appropriately. Otherwise they are calculated
# from the data read in.
set nicVar(graph,user,yMin)		0.0
set nicVar(graph,user,yMax)		300.0
set nicVar(graph,user,xMin)		280.0
set nicVar(graph,user,xMax) 	330.0

# Set the number of tic marks for each scale
set nicVar(graph,yTics)			6
set nicVar(graph,xTics)			10	

# choices for the x-axis placement: bottom, middle, or top
set nicVar(graph,xaxisLoc)		"bottom"	

#
# End User section
#
########################################################################

########################################################################
#
# To change program for a different language edit these variables
#
set nicVar(text,logo1)			"NMR Integration Calculations"
set nicVar(text,input_dir)		"Input Directory:"
set nicVar(text,browse)			"Browse"
set nicVar(text,output_file)	"Results File:"
set nicVar(text,lb)				"Data Lower Bound:"
set nicVar(text,ub)				"Data Upper Bound:"
set nicVar(text,calc)			"Calculate"
set nicVar(text,exit)			"Exit"
set nicVar(text,msg1)			"Calculating..."
set nicVar(text,msg2)			"Click result line to display plot"
set nicVar(text,file)			"File"
set nicVar(text,vol)			"Volume"
set nicVar(text,name)			"Name"
set nicVar(text,tp)				"TotalPoints"
set nicVar(text,used)			"Used"
set nicVar(text,help)			"Help"
set nicVar(text,next_graph)		"Next Graph"
set nicVar(text,prev_graph)		"Prev Graph"
set nicVar(text,save)			"Save Postscript"
set nicVar(text,xLabText)		"ppm"
set nicVar(text,yLabText)		"Intensity"	

##########################################################################


###############################################################################
#
# Here starts the Main Program
#
proc nic_main {} {
	global nicVar

	option add *background $nicVar(main,bg)
	option add *foreground $nicVar(main,fg)
	option add *highlightBackground $nicVar(main,bg)
	option add *highlightColor $nicVar(main,bg)
	option add *listing.background $nicVar(main,listing,bg)
	option add *listing.font $nicVar(main,fixed_font)
	option add *entry.background $nicVar(main,entry,bg)

	set f $nicVar(main,name)
	toplevel $f
	wm title $f "$nicVar(PROG) - $nicVar(VERSION)"
	wm geometry $f $nicVar(main,geometry)

	# draw the main window
	show_menubar $nicVar(main,name)
	show_logo $nicVar(main,name)
	show_parms $nicVar(main,name).parms
	show_buts $nicVar(main,name).buts
	label $nicVar(main,name).msg
	pack $nicVar(main,name).msg
	show_results_panel $nicVar(main,name).pResults

	# wait until the user exits 
	tkwait window $nicVar(main,name)
}

###############################################################################
#
# create menubar
#
proc show_menubar { f } {
	global nicVar

	set mb $f.mbar
	frame $mb -relief raised -bd 2
	frame $f.dummy
	pack $mb -side top -fill x
	set f $mb

	menubutton $f.file -padx 10 -text $nicVar(text,file) \
		-underline 0 -menu $f.file.menu
	menubutton $f.help -padx 10 -text $nicVar(text,help) \
		-underline 0 -menu $f.help.menu
	
	menu $f.file.menu
	$f.file.menu add command -label "About the program..." \
		-command "pop_authors $f.authors"
	$f.file.menu add separator
	$f.file.menu add command -label "Exit" -command "exit"
	
	menu $f.help.menu
	$f.help.menu add command -label "see $nicVar(WEB_HOME)"
	
	pack $f.file $f.help -side left
	tk_menuBar $f $f.file $f.help
	focus $f
}

###############################################################################
#
# show main logo image in starting window 
#
proc show_logo { f } {
	global nicVar 

	set myTitle $nicVar(text,logo1)
	set text "$myTitle - $nicVar(VERSION) $nicVar(VERSION_DATE)"

	if [file exists $nicVar(HEADER_IMAGE)] {
    	canvas $f.header -height 115 -bg white -width 540
		pack $f.header -fill both -expand true -pady 19
		set img [image create photo -file $nicVar(HEADER_IMAGE)]
		$f.header create image 0 0 -image $img -anchor nw
		$f.header create text 290 100 -text $text \
			-justify center -anchor n -fill red
	} else {
    	canvas $f.header -height 25 -bg white -width 540
		pack $f.header -fill both -expand true -pady 19
		$f.header create text 290 10 -text $text \
			-justify center -anchor n -fill red
	}
}

###############################################################################
#
# show parameters for computing volumes of each file 
#
proc show_parms {f} {
	global nicVar

	set labelLen 20
	set padx 10

	frame $f
	pack $f 

	frame $f.files
	pack $f.files -pady 20

	if ![file exists $nicVar(input_dir)] {
		set nicVar(input_dir) ""
		set nicVar(output_file) ""
	}
	u_file_entry $f.files.inputdir $nicVar(text,input_dir) \
		$labelLen nicVar(input_dir) $nicVar(main,file_len) $padx {} 2
	pack $f.files.inputdir

	u_file_entry $f.files.output $nicVar(text,output_file) \
		$labelLen nicVar(output_file) $nicVar(main,file_len) $padx {} 0
	pack $f.files.output

	frame $f.parms
	pack $f.parms -pady 10

	parm_entry $f.parms.lb $nicVar(text,lb) $labelLen nicVar(lb) 10 ""
	pack $f.parms.lb
	parm_entry $f.parms.ub $nicVar(text,ub) $labelLen nicVar(ub) 10 ""
	pack $f.parms.ub
}

###############################################################################
#
# show main window buttons 
#
proc show_buts {f} {
	global nicVar

	frame $f
	pack $f -pady 20 
	button $f.calcBut -text $nicVar(text,calc) -command "calc_volumes"
	button $f.quitBut -text $nicVar(text,exit) -command "exit"
	pack $f.calcBut $f.quitBut -side left
}

######################################################################
#
# create results panel in the main window 
#
proc show_results_panel { f } {
	global nicVar 

	frame $f
	pack $f -pady 10 -expand true -fill both -padx 10
	set header " Results"
	label $f.label -text "$header"
	pack $f.label -anchor w 

	set nicVar(items) [listbox $f.listing \
		-setgrid false -selectmode browse \
		-yscrollcommand "$f.scroll set"]
	scrollbar $f.scroll -command "$f.listing yview"
	pack $f.scroll -side right -fill y
	pack $f.listing -side left -fill both -expand true
	bind $nicVar(items) <ButtonRelease-1> \
		[list results_click $nicVar(items) %y]

	pack $f -pady 10 -side top -fill both -expand true
}

########################################################################
# user selected an item in the results panel 
#
proc results_click { list y } {
	global nicVar

	# no items to select
    if { $nicVar(data,fileNum) == 0 } { return }

    # find out which item was selected, index starting at 1
    set iNum [$nicVar(items) nearest $y]
	incr iNum
    set nicVar(graph,$iNum,file) $iNum

    # draw this window 
	do_plot_window $iNum
}

########################################################################
# show results plotting window 
#
proc do_plot_window { iNum } {
	global nicVar

	# create a new window
	set nicVar(graph,$iNum,name) $nicVar(main,name)-$iNum
	set f $nicVar(graph,$iNum,name)
	toplevel $f
	wm title $f $nicVar(graph,$iNum,name)
	wm geometry $f $nicVar(graph,geometry)

	# create histogram canvas with movable elements
	set nicVar(graph,$iNum,canvas) $f.c
	canvas $nicVar(graph,$iNum,canvas) -bg $nicVar(graph,bg)
	pack $nicVar(graph,$iNum,canvas)
	$nicVar(graph,$iNum,canvas) bind movable <Button-1> {Mark %x %y %W}
	$nicVar(graph,$iNum,canvas) bind movable <B1-Motion> {Drag %x %y %W}

	# some buttons to allow user to control the window
	frame $f.buttons
	pack $f.buttons
	button $f.buttons.1 -text $nicVar(text,next_graph) \
		-command "next_graph $f $iNum 1"
	button $f.buttons.2 -text $nicVar(text,prev_graph) \
		-command "next_graph $f $iNum -1"
	button $f.buttons.3 -text $nicVar(text,save) \
		-command "pop_save_ps .ps $iNum"
	button $f.buttons.4 -text $nicVar(text,exit) -command "destroy $f"
	pack $f.buttons.1 $f.buttons.2 $f.buttons.3 $f.buttons.4 \
		-side left -pady 10

	# show the plot
	set nicVar(graph,$iNum,resizeEvent) 0
	bind $nicVar(graph,$iNum,canvas) <Configure> "resize_event $iNum"
}

########################################################################
# User resizes a plotting window 
#
proc resize_event { iNum } {
	global nicVar 

	if { $nicVar(graph,$iNum,resizeEvent) == 1 } { return }

	set f $nicVar(graph,$iNum,name)
	set nicVar(graph,$iNum,geometry) [wm geometry $f]
	next_graph $f $iNum 0
	set nicVar(graph,$iNum,resizeEvent) 1
	update
	set nicVar(graph,$iNum,resizeEvent) 0
}

########################################################################
# user wants to draw the next or previous graph 
#
proc next_graph { f iNum inc } {
	global nicVar

	# decide what graph number that will be
	if {$inc < 0 && $nicVar(graph,$iNum,file) <= 1} {
		set nicVar(graph,$iNum,file) $nicVar(data,fileNum)
	} else {
		if {$inc > 0 && $nicVar(graph,$iNum,file) >= $nicVar(data,fileNum)} {
			set nicVar(graph,$iNum,file) 1
		} else {
			set nicVar(graph,$iNum,file) [expr $nicVar(graph,$iNum,file) + $inc]
		}
	}
	set n $nicVar(graph,$iNum,file)

	# draw the graph
	do_graph $f $n 
}

########################################################################
# draw plot in the results window 
#
proc do_graph { f iNum } {
	global nicVar

	# decide upon some attributes for the graph
	foreach item {yMin yMax xMin xMax} {
    	if [info exists nicVar(graph,user,$item)] { 
			set nicVar(graph,$item) $nicVar(graph,user,$item)
		} else {
			set nicVar(graph,$item) $nicVar(data,$iNum,$item)
		}
	}

    if {$nicVar(graph,xaxisLoc) == "bottom"} {
		set nicVar(graph,yMin) 0
	}
    if {$nicVar(graph,xaxisLoc) == "middle"} {
		set yTmp [expr 0 - $nicVar(graph,yMin)]
		if {$yTmp > $nicVar(graph,yMax)} { set nicVar(graph,yMax) $yTmp }
		if {$yTmp < $nicVar(graph,yMax)} { set nicVar(graph,yMin) [expr 0 - $nicVar(graph,yMax)] }
	}

	# make a title
	set outline [format "$nicVar(text,file) = %s    $nicVar(text,vol)=%-10.2f" \
		$nicVar(data,$iNum,path) $nicVar(data,$iNum,volume) ]
	set nicVar(graph,titleText) $outline

	# ok, draw it!
	draw_histo $f $iNum
}

###############################################################################
###############################################################################

###############################################################################
#
# update histogram canvas, erase the old one and draw the elements of the
# new one.
#
proc draw_histo { wind iNum } {
	global nicVar 

	# update histogram variables
	set nicVar(graph,$iNum,geometry) [wm geometry $wind] 
	set newHeight [lindex [split $nicVar(graph,$iNum,geometry) x+-] 1]
	set nicVar(graph,width) [lindex [split $nicVar(graph,$iNum,geometry) x+-] 0]
	set nicVar(graph,height) [expr $newHeight - 60]

	# update canvas 
	$wind.c configure -width $nicVar(graph,width)
	$wind.c configure -height $nicVar(graph,height)

	# erase old histogram
	set items [$wind.c find all]
	foreach item $items {
		$wind.c delete $item
	}
	update

	draw_title $wind.c
	draw_axis $wind.c
	draw_xScale $wind.c
	draw_yScale $wind.c
	draw_data $wind.c $iNum \
		$nicVar(graph,line1_color) $nicVar(graph,line1_shape) $nicVar(graph,line1_size)

}

###############################################################################
#
# draw axis on screen 
# 
proc draw_axis { fc } {
	global nicVar

	# find axis lengths
	set nicVar(graph,yLen) [expr $nicVar(graph,height) - $nicVar(graph,yStart) - $nicVar(graph,yStart)]
	set nicVar(graph,xLen) [expr $nicVar(graph,width) - $nicVar(graph,xStart) - $nicVar(graph,xStart)]

	# find yOrigin for xaxis
	set nicVar(graph,yEnd) [expr $nicVar(graph,yStart) + $nicVar(graph,yLen)]
	set nicVar(graph,xEnd) [expr $nicVar(graph,xStart) + $nicVar(graph,xLen)]
	switch $nicVar(graph,xaxisLoc) {
		top { set nicVar(graph,yOrigin) $nicVar(graph,yStart) }
		bottom { set nicVar(graph,yOrigin) $nicVar(graph,yEnd) }
		middle { set nicVar(graph,yOrigin) [expr ($nicVar(graph,yStart)+$nicVar(graph,yEnd)) / 2] }
	}

	# y axis
	$fc create line $nicVar(graph,xStart) $nicVar(graph,yStart) \
		$nicVar(graph,xStart) $nicVar(graph,yEnd) -width 1 -fill $nicVar(graph,axis_color)

	# x axis
	$fc create line $nicVar(graph,xStart) $nicVar(graph,yOrigin) \
		$nicVar(graph,xEnd) $nicVar(graph,yOrigin) -width 1 -fill $nicVar(graph,axis_color)
}

###############################################################################
#
# draw plot title 
# 
proc draw_title { fc } {
	global nicVar

	set xTitle [expr $nicVar(graph,width) / 2]
	set yTitle [expr $nicVar(graph,yStart) / 3]

	$fc create text $xTitle $yTitle \
		-text $nicVar(graph,titleText) -fill $nicVar(graph,title_color) -tag movable
}

###############################################################################
#
# draw x axis scale 
# 
proc draw_xScale { fc } {
	global nicVar 

	set xLabInc [expr 1 + ($nicVar(graph,xMax) - $nicVar(graph,xMin)) / $nicVar(graph,xTics)]
	set xLabStart [expr $nicVar(graph,xMin) + $xLabInc]
	set xTicTop [expr $nicVar(graph,yOrigin) - 5]
	set xTicBot [expr $nicVar(graph,yOrigin) + 5]

	if {[expr $xLabStart + $xLabInc] > $nicVar(graph,xMax)} { return }

	set xPixelRatio [expr $nicVar(graph,xLen) / double($nicVar(graph,xMax) - $nicVar(graph,xMin))]
	set xInc [expr $xLabStart - $nicVar(graph,xMin)]
	set px [expr $nicVar(graph,xStart) + $xInc * $xPixelRatio]
	set py [expr $nicVar(graph,yOrigin) + $nicVar(graph,xScaleOffset)]
	set xi $xLabStart 

	$fc create line $px $xTicTop $px $xTicBot -fill $nicVar(graph,axis_color)
	set sVal [format "%.4g" $xi]
	$fc create text $px $py -text $sVal -fill $nicVar(graph,axis_color) -tag movable

	set xEnd [expr $nicVar(graph,xStart) + $nicVar(graph,xLen)]
	set xi [expr $xi + $xLabInc]
	set px [expr $px + $xLabInc * $xPixelRatio]

	while {$px < $nicVar(graph,xEnd)} {
		$fc create line $px $xTicTop $px $xTicBot -fill $nicVar(graph,axis_color)
		set sVal [format "%.4g" $xi]
		$fc create text $px $py -text $sVal -fill $nicVar(graph,axis_color) -tag movable
		set xi [expr $xi + $xLabInc]
		set px [expr $px + $xLabInc * $xPixelRatio]
	}

	set px [expr $nicVar(graph,xLen) + $nicVar(graph,xStart)]
	set py [expr $py + 20]
	$fc create text $px $py -text $nicVar(text,xLabText) \
		-fill $nicVar(graph,axis_color) -tag movable
}

###############################################################################
#
# draw y axis scale on graph 
# 
proc draw_yScale { fc } {
	global nicVar 

	set yTicLeft [expr $nicVar(graph,xStart) - 5]
	set yTicRight [expr $nicVar(graph,xStart) + 5]

	set px [expr $nicVar(graph,xStart) - 40]

	if {$nicVar(graph,yTics) <= 0} { return }
	if {$nicVar(graph,yTics) == 1} { 
		$fc create line $yTicLeft $nicVar(graph,yOrigin) $yTicRight $yOrigin \
			-fill $aColor
		set sVal [format "%5.3f" 0.000]
		$fc create text $px $nicVar(graph,yOrigin) -text $sVal -fill $nicVar(graph,axis_color) \
			-tag movable
		return
	}

	set nTics [expr $nicVar(graph,yTics) - 1]
	set yInc [expr ($nicVar(graph,yMax) - $nicVar(graph,yMin)) / $nTics]
	set yi $nicVar(graph,yMin)
	set pInc [expr double($nicVar(graph,yLen)) / $nTics]
	for {set i 0} {$i <= $nTics} {incr i} {
		set py [expr $nicVar(graph,yStart) + $nicVar(graph,yLen) - round(0.50 + $pInc * $i)]
		$fc create line $yTicLeft $py $yTicRight $py -fill $nicVar(graph,axis_color)
		set sVal [format "%5.3f" $yi]
		$fc create text $px $py -text $sVal -fill $nicVar(graph,axis_color) -tag movable
		set yi [expr $yi + $yInc]
	}

	set py [expr $nicVar(graph,yStart) - $nicVar(graph,yLabOffset)]
	$fc create text $px $py -text $nicVar(text,yLabText) \
		-fill $nicVar(graph,axis_color) -tag movable
}

###############################################################################
#
# draw data points on the screen 
# 
proc draw_data { fc iNum aColor shape size } {
	global nicVar

	set lbFlag 0
	set ubFlag 0

	if {$nicVar(data,$iNum,nPoints) == 0} {return}

	set xPixelRatio [expr $nicVar(graph,xLen) / double($nicVar(graph,xMax) - $nicVar(graph,xMin))]
	set yPixelRatio [expr $nicVar(graph,yLen) / double($nicVar(graph,yMax) - $nicVar(graph,yMin))]

	for {set j 0} {$j < $nicVar(data,$iNum,nPoints)} {incr j} {
		set px [expr $nicVar(graph,xStart) + ($nicVar(data,$iNum,x,$j) - $nicVar(graph,xMin)) * $xPixelRatio]
		if {$px >= $nicVar(graph,xStart) && $px <= [expr $nicVar(graph,xStart) + $nicVar(graph,xLen)]} {
			set py [expr $nicVar(graph,yOrigin) - $nicVar(data,$iNum,y,$j) * $yPixelRatio]
			$fc create $shape [expr $px-$size] [expr $py-$size] \
				[expr $px+$size] [expr $py+$size] -outline $aColor -width 1
			if {$lbFlag == 0 && $j > 0} {
				if { $nicVar(data,$iNum,x,$j) >= $nicVar(lb) } {
					set lbFlag 1
					draw_bound $fc $px $nicVar(lb)
				}
			}
			if {$ubFlag == 0 && $j > 0} {
				set last [expr $nicVar(data,$iNum,nPoints) - 1]
				if { $nicVar(data,$iNum,x,$j) > $nicVar(ub) || $j == $last } {
					set ubFlag 1
					draw_bound $fc $oldpx $nicVar(ub)
				}
			}
			if [info exists oldpx] {
				set k [expr $j - 1]
				set q1 $nicVar(data,$iNum,x,$k)
				set q2 [expr $nicVar(data,$iNum,x,$j) - 1]
				if {$q1 == $q2} {
					$fc create line $oldpx $oldpy $px $py \
						-width 1 -fill $aColor
				}
			}
			set oldpx $px
			set oldpy $py
		}
	}
}

###############################################################################
#
# draw bound line (cutoffs) on the graph
#
proc draw_bound { fc px bound } {
	global nicVar

	$fc create line $px $nicVar(graph,yStart) $px $nicVar(graph,yEnd) \
		-width 1 -fill $nicVar(graph,bound_color)
	set p1 [expr $px - 1]
	set p2 [expr $nicVar(graph,yStart) - $nicVar(graph,yLabOffset)]
	set sVal [format "%5.1f" $bound]
	$fc create text $p1 $p2 -text $sVal \
		-fill $nicVar(graph,bound_color) -tag movable
}

###############################################################################
#
# Allows user to drag elements of the histogram (eg, titles) to more
# appropriate places
#
proc Mark {x y w } {
	global state
	set state($w,obj) [$w find closest $x $y]
	set state($w,x) $x
	set state($w,y) $y
}
proc Drag {x y w} {
	global state
	set dx [expr $x - $state($w,x)]
	set dy [expr $y - $state($w,y)]
	$w move $state($w,obj) $dx $dy
	set state($w,x) $x
	set state($w,y) $y
}

########################################################################
########################################################################
########################################################################

######################################################################
#
# misc TCL utilities
#
######################################################################

######################################################################
#
# parameter field entry
#
proc parm_entry {f labName labLen entVar entLen command} {
	frame $f
	label $f.label -text $labName -width $labLen -anchor w -relief flat
	entry $f.entry -relief sunken -textvar $entVar -width $entLen
	bind $f.entry <Return> $command
	pack $f.label -side left
	pack $f.entry -fill x -expand true
}

######################################################################
#
# generic command field entry
#
proc command_entry {name label width command args} {
	frame $name
	label $name.label -text $label -width $width -anchor w
	eval {entry $name.entry -relief sunken } $args
	pack $name.label -side left
	pack $name.entry -side right -fill x -expand true
	bind $name.entry <Return> $command
	return $name.entry
}

######################################################################
#
# save graphics to a postscript file
#
proc pop_save_ps { f iNum } {
	global nicVar 

	toplevel $f 
	label $f.nop1
	label $f.nop2

	set nicVar(fplotc) $nicVar(graph,$iNum,canvas)
	set savecmd {$nicVar(fplotc) postscript -file ./$nicVar(psFname); exec chmod 600 ./$nicVar(psFname)}

	command_entry $f.ps "Save to postscript file: " 26 \
		"$savecmd; destroy $f" -textvar nicVar(psFname) -width 20 
	pack $f.nop1 $f.ps $f.nop2 -side top

	button $f.but -text "Cancel" -command "destroy $f"
	pack $f.but -side top
}

######################################################################
#
# purpose: to call a tk routine to search for a directory
#
proc u_dir_browse { dir } {
	global nicVar

	set chosen [tk_chooseDirectory]
	if { $chosen != "" } {
		set currentpwd [pwd]
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $dir $shortchosen

		# set output file 
		set nicVar(output_file)	"[file tail $nicVar(input_dir)]$nicVar(fileSuffix)"
	}
}

######################################################################
#
# generic file entry widget
#
proc u_file_entry { f title tWidth tVar eWidth padx cmd browse } {
	global nicVar 

    pack [frame $f] -fill x -expand true -padx 1
    pack [frame $f.a] -side left
    pack [frame $f.b] -side left
    pack [frame $f.c] -side left 
	label $f.a.label -text $title -anchor e -width $tWidth
	pack $f.a.label -side right
	entry $f.b.entry -bd 2 -textvariable $tVar -width $eWidth -relief sunken
#	$f.b.entry config -cursor {xterm white}
	bind $f.b.entry <Return> $cmd
	pack $f.b.entry -side left -anchor nw
	if { $browse > 0 } {
		if { $browse == 1 } {
			button $f.c.browse -text $nicVar(text,browse) \
				-command "u_file_browse $tVar" 
		} else {
			button $f.c.browse -text $nicVar(text,browse) \
				-command "u_dir_browse $tVar" 
		}
		pack $f.c.browse -side left -padx 8 -anchor nw
	}
}

###############################################################################
#
# About the program window
#
proc pop_authors { f } {
	global nicVar 

	toplevel $f 

	label $f.vers -text $nicVar(text,logo1)
	label $f.date -text "$nicVar(VERSION) $nicVar(VERSION_DATE)"
	label $f.nop1
	label $f.aut \
		-text "Authors: Robert Boyko / Olga Baryshnikova / Brian Sykes"
	label $f.nop2
	label $f.msg1 \
		-text " Note: This project is funded by the Canadian Institutes of Health Research"

	label $f.nop3
	label $f.msg3 -text " Website - \
	 $nicVar(WEB_HOME) \n \
	 Email - webmaster@biochem.ualberta.ca"
	button $f.buts -text "Ok" -command "destroy $f"
	label $f.nop4
	pack $f.vers $f.date $f.nop1 $f.aut $f.nop2 $f.msg1 
	pack $f.nop3 $f.msg3 $f.nop4
	pack $f.buts -side top
}

########################################################################
########################################################################

########################################################################
# reads one input file
#
proc read_data { iNum fn } {
	global nicVar 

	if {[ catch { set dataFile [open $fn r ] } err]} {
		error "Cannot read data file '$fn'"
	}

	set nPoints 0
	set vol 0.0

	# set max and min values 
	set nicVar(data,$iNum,yMax) -1000000 
	set nicVar(data,$iNum,yMin) 1000000

	while {[gets $dataFile line] >= 0} {
		set nd [scan $line "%f %f " x y]

		# For now assume all 2 column rows are data, x values are sorted 
		if { $nd == 2 } {
			set nicVar(data,$iNum,x,$nPoints) $x
			set nicVar(data,$iNum,y,$nPoints) $y

			# Keep track of max and min data values
			if { [expr $y > $nicVar(data,$iNum,yMax)] } { 
				set nicVar(data,$iNum,yMax) $y
			}
			if { [expr $y < $nicVar(data,$iNum,yMin)] } { 
				set nicVar(data,$iNum,yMin) $y
			}
			set nicVar(data,$iNum,xMin) $nicVar(data,$iNum,x,0)
			set nicVar(data,$iNum,xMax) $x
			incr nPoints 
		}
	}

	# Here is how many points we read in
	set nicVar(data,$iNum,nPoints) $nPoints

	close $dataFile
}

###############################################################################
#
# calculate volume of one file of data
#
proc calc_data { iNum fn fout } {
	global nicVar

	set prev 0
	set totVol 0.0
	set nicVar(data,$iNum,path) $fn
	set nicVar(data,$iNum,fn) [file tail $fn]
	set nicVar(data,$iNum,cPoints) 0

	for {set i 1} {$i < $nicVar(data,$iNum,nPoints)} {incr i} {
		if { $nicVar(data,$iNum,x,$prev) >= $nicVar(lb) && 
			$nicVar(data,$iNum,x,$i) <= $nicVar(ub) } { 

			if { $nicVar(data,$iNum,y,$i) > $nicVar(data,$iNum,y,$prev)} {
				set base $nicVar(data,$iNum,y,$prev)
			} else {
				set base $nicVar(data,$iNum,y,$i)
			}

			set wd [expr $nicVar(data,$iNum,x,$i) - $nicVar(data,$iNum,x,$prev)]
			set ht [expr $nicVar(data,$iNum,y,$i) - $nicVar(data,$iNum,y,$prev)]

			set v1 [expr abs($wd * $ht * 0.5)]
			set v2 [expr abs($wd * $base)]
			set incVol [ expr $v1 + $v2 ]

			set totVol [expr $totVol + $incVol]
			incr nicVar(data,$iNum,cPoints)
		}
		incr prev
	}
	set nicVar(data,$iNum,volume) $totVol
	set outline [format "%4d %20s %10.2f" \
		$nicVar(data,fileNum) $nicVar(data,$iNum,fn) $totVol ]
	if {$nicVar(rFlag) == 1 } {
		set extra [format "%13d %6d" \
			[expr $nicVar(data,$iNum,nPoints)-1] $nicVar(data,$iNum,cPoints) ]
		set outline "$outline $extra"
	}
	puts $fout $outline
	$nicVar(items) insert end $outline
}


########################################################################
# process and integrate all spectral files in a directory of data 
#
proc calc_volumes {} {
	global nicVar

	set header "$nicVar(text,file)                 $nicVar(text,name)       $nicVar(text,vol)"
	if {$nicVar(rFlag) == 1 } {
		set header "$header  $nicVar(text,tp)   $nicVar(text,used)"
	}
	set fout [open $nicVar(output_file) w]
	puts $fout "$nicVar(text,input_dir) $nicVar(input_dir)"
	puts $fout "$nicVar(text,lb) $nicVar(lb) $nicVar(text,ub) $nicVar(ub)"
	puts $fout ""
	puts $fout $header 
	puts $fout ""
	set nicVar(data,fileNum) 0

	$nicVar(main,name).msg configure -text $nicVar(text,msg1) 
	update;

	$nicVar(items) delete 0 end
	$nicVar(main,name).pResults.label configure -text $header \
		-font $nicVar(main,fixed_font)

	# process and integrate each file found
	set files [glob -nocomplain $nicVar(input_dir)/*]
	foreach f [lsort $files] {
		if [file readable $f] {
			incr nicVar(data,fileNum)
			read_data $nicVar(data,fileNum) $f 
			calc_data $nicVar(data,fileNum) $f $fout
		}
	}
	close $fout
	$nicVar(main,name).msg configure -text $nicVar(text,msg2)
}

########################################################################
########################################################################
	
wm withdraw .
nic_main
destroy .
